<?php
// Own namespace
namespace Org\Mxchange\CoreFramework\Resolver\Command;

// Import framework stuff
use Org\Mxchange\CoreFramework\Bootstrap\FrameworkBootstrap;
use Org\Mxchange\CoreFramework\Command\Commandable;
use Org\Mxchange\CoreFramework\Command\InvalidCommandException;
use Org\Mxchange\CoreFramework\Factory\Object\ObjectFactory;
use Org\Mxchange\CoreFramework\Request\Requestable;
use Org\Mxchange\CoreFramework\Resolver\BaseResolver;
use Org\Mxchange\CoreFramework\Utils\String\StringUtils;

// Import SPL stuff
use \InvalidArgumentException;
use \UnexpectedValueException;

/**
 * A generic command resolver class
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2021 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
abstract class BaseCommandResolver extends BaseResolver {
	/**
	 * Command name
	 */
	private $commandName = '';

	/**
	 * Protected constructor
	 *
	 * @param	$className	Name of the class
	 * @return	void
	 */
	protected function __construct (string $className) {
		// Call parent constructor
		parent::__construct($className);
	}

	/**
	 * Setter for command name
	 *
	 * @param	$commandName	Last validated command name
	 * @return	void
	 */
	protected final function setCommandName (string $commandName) {
		$this->commandName = $commandName;
	}

	/**
	 * Getter for command name
	 *
	 * @return	$commandName	Last validated command name
	 */
	public final function getCommandName () {
		return $this->commandName;
	}

	/**
	 * "Loads" a given command and instances it if not yet cached
	 *
	 * @param	$commandName				A command name we shall look for
	 * @return	$commandInstance			A loaded command instance
	 * @throws	InvalidCommandException		Thrown if even the default
	 *										command class is missing (bad!)
	 */
	protected function loadCommand ($commandName) {
		// Init command instance
		$commandInstance = NULL;

		// Create class name
		$className = sprintf(
			'%s\%s%sCommand',
			$this->getNamespace(),
			$this->getCapitalizedClassPrefix(),
			StringUtils::convertToClassName($commandName)
		);

		// Create command class name
		$this->setClassName($className);

		// Is this class loaded?
		if (!class_exists($this->getClassName())) {
			// Class not found, so throw an exception
			throw new InvalidCommandException(array($this, $commandName), self::EXCEPTION_INVALID_COMMAND);
		}

		// Initiate the command
		$commandInstance = ObjectFactory::createObjectByName($this->getClassName(), [$this]);

		// Return the result
		return $commandInstance;
	}

	/**
	 * Returns an command instance for a given request class or null if
	 * it was not found
	 *
	 * @param	$requestInstance	An instance of a Requestable class
	 * @return	$commandInstance	An instance of the resolved command
	 * @throws	InvalidCommandException		Thrown if $commandName is invalid
	 * @throws	UnexpectedValueException	Thrown if $commandInstance is an invalid instance
	 */
	public function resolveCommandByRequest (Requestable $requestInstance) {
		// Init variables
		$commandName = $requestInstance->getRequestElement('command');
		$commandType = FrameworkBootstrap::getRequestTypeFromSystem();
		$commandInstance = NULL;

		// Is the command empty? Then fall back to default command
		if (empty($commandName)) {
			// Fall back to default command
			$commandName = FrameworkBootstrap::getConfigurationInstance()->getConfigEntry(sprintf('default_%s_command', $commandType));
		}

		// Check if command is valid
		if ($this->isCommandValid($this->getNamespace(), $commandName) === false) {
			// This command is invalid!
			throw new InvalidCommandException(array($this, $commandName), self::EXCEPTION_INVALID_COMMAND);
		}

		// Get the command
		$commandInstance = $this->loadCommand($commandName);

		// And validate it
		if ((!is_object($commandInstance)) || (!$commandInstance instanceof Commandable)) {
			// This command has an invalid instance!
			throw new UnexpectedValueException(sprintf('commandInstance for commandName=%s is not object (%s) or does not implement Commandable.', $commandName, gettype($commandInstance)), self::EXCEPTION_INVALID_COMMAND);
		}

		// Set last command
		$this->setResolvedInstance($commandInstance);

		// Init template engine
		$commandInstance->initTemplateEngine($commandType);

		// Return the resolved command instance
		return $commandInstance;
	}

	/**
	 * Resolves the command by its direct name and returns an instance of its class
	 *
	 * @param	$namespace			Namespace to look in
	 * @param	$commandName		The direct command name we shall resolve
	 * @return	$commandInstance	An instance of the command class
	 * @throws	InvalidCommandException		Thrown if $commandName is invalid
	 */
	public function resolveCommand (string $namespace, string $commandName) {
		// Is a action set?
		if (empty($namespace)) {
			// Then thrown an exception here
			throw new InvalidArgumentException('Parameter "namespace" is empty');
		} elseif (empty($commandName)) {
			// Then thrown an exception here
			throw new InvalidArgumentException('Parameter "commandName" is empty');
		}

		// Initiate the instance variable
		$commandInstance = NULL;
		$commandType = FrameworkBootstrap::getRequestTypeFromSystem();

		// Is the command empty? Then fall back to default command
		if (empty($commandName)) {
			// Init default command
			$commandName = FrameworkBootstrap::getConfigurationInstance()->getConfigEntry(sprintf('default_%s_command', $commandType));
		}

		// Check if command is valid
		if ($this->isCommandValid($namespace, $commandName) === false) {
			// This command is invalid!
			throw new InvalidCommandException(array($this, $commandName), self::EXCEPTION_INVALID_COMMAND);
		}

		// Get the command
		$commandInstance = $this->loadCommand($commandName);

		// Init template engine
		$commandInstance->initTemplateEngine($commandType);

		// Return the instance
		return $commandInstance;
	}

	/**
	 * Checks whether the given command is valid
	 *
	 * @param	$namespace		Namespace to look in
	 * @param	$commandName	The default command we shall execute
	 * @return	$isValid		Whether the given command is valid
	 * @throws	InvalidArgumentException	Thrown if given command is not set
	 */
	protected function isCommandValid (string $namespace, string $commandName) {
		// Is namespace and command name set?
		if (empty($namespace)) {
			// Then thrown an exception here
			throw new InvalidArgumentException('Parameter "namespace" is empty');
		} elseif (empty($commandName)) {
			// Then thrown an exception here
			throw new InvalidArgumentException('Parameter "commandName" is empty');
		}

		// By default nothing shall be valid
		$isValid = false;

		// Create the full class name
		$className = sprintf(
			'%s\%s%sCommand',
			$namespace,
			$this->getCapitalizedClassPrefix(),
			StringUtils::convertToClassName($commandName)
		);

		// Now, let us create the full name of the command class
		$this->setClassName($className);

		// Is this class already loaded?
		if (class_exists($this->getClassName())) {
			// This class does exist. :-)
			$isValid = true;
		}

		// Set command name
		$this->setCommandName($commandName);

		// Return the result
		return $isValid;
	}

}
